]> ruderich.org/simon Gitweb - config/dotfiles.git/blobdiff - lib.sh
lib.sh: Add support for FreeBSD to os().
[config/dotfiles.git] / lib.sh
diff --git a/lib.sh b/lib.sh
index fbef907d330bd62aec6bf67a5d0b4c237c267159..206f9954a3841041da04e8b4417dccf497b889dc 100644 (file)
--- a/lib.sh
+++ b/lib.sh
@@ -4,19 +4,23 @@
 
 
 # csh gives the error "Unknown colorls variable `su'." when used with newer
-# options supported by zsh or GNU ls.
+# options supported by Zsh or GNU ls.
 unset LS_COLORS
 
 
 # Check if the given program is installed. Returns 0 if it exists, 1
 # otherwise; so it can be used in if.
 installed() {
-    which $1 | grep -E '^/' > /dev/null
+    [ x`which $1 2>&1 | cut -c 1` = x/ ] || return 1
 }
 
# Prints the current OS. Supported are Debian (debian), Gentoo (gentoo) and
# Mac OS X (darwin) at the moment. If an unsupported OS is used an error is
# printed.
 os() {
     if [ -f /etc/debian_version ]; then
         echo debian
@@ -24,13 +28,17 @@ os() {
         echo gentoo
     elif [ x`uname` = xDarwin ]; then
         echo darwin
+    elif [ x`uname` = xSunOS ]; then
+        echo sun
+    elif [ x`uname` = xFreeBSD ]; then
+        echo freebsd
     else
         echo unsupported OS! >&2
         return 1
     fi
 }
 
-# Creates a symbolic link for file $1 in dirname of $2 with name of basenmae
+# Creates a symbolic link for file $1 in dirname of $2 with name of basename
 # $2.
 #
 # `./link.sh example ~/.examplerc` creates a symbolic link to example
@@ -38,8 +46,9 @@ os() {
 link() {
     # Get all necessary paths.
     pwd=`pwd`
-    base=`dirname "$2"`
-    source=`echo -n "$pwd/$1" | sed "s|$base/||"`
+    base=`echo "$2" | sed "s|\~|$HOME|"` # expand ~, some sh don't do it
+    base=`dirname "$base"`
+    source=`echo "$pwd/$1" | sed "s|$base/||"`
     target=`basename "$2"`
 
     # Go to the directory where the link is going to be created.
@@ -47,13 +56,14 @@ link() {
 
     # Abort if the target file exists and is no symbolic link. Prevents
     # overwriting real files.
-    if [ -e "$target" -a ! -h "$target" ]; then
+    if [ \( -f "$target" -a ! -h "$target" \) -o \
+         \( -s "$target" -a ! -h "$target" \) ]; then
         echo "link(): target '$target' exists already and is no symbolic link!" >&2
         exit 1
     fi
 
-    # Make sure the source exists.
-    if [ ! -e "$source" ]; then
+    # Make sure the source exists (is file, directory or link).
+    if [ ! -f "$source" -a ! -d "$source" -a ! -h "$source" ]; then
         echo "link(): source '$source' doesn't exist!" >&2
         exit 1
     fi
@@ -71,7 +81,7 @@ link() {
 
 # Write a warning to $1 to make clear it should not be modified. $2 is the
 # source for the generated file. Also print a message to stdout that the file
-# $1 was generated from $2 using the command $3.
+# $1 was generated from $2 using the command $3 with options $4.
 warning() {
     echo "###################################" > $1
     echo "# WARNING! DO NOT EDIT THIS FILE! #" >> $1
@@ -80,19 +90,29 @@ warning() {
     echo "# It was generated from $2 on `date`." >> $1
     echo >> $1
 
+    # Display given options if there were any (Zsh has a problem with $options
+    # as variable name).
+    option=
+    if [ -n "$4" ]; then
+        option=" with options '$4'"
+    fi
     # Write message to stdout.
-    echo "$3: generating '$1' from '$2'"
+    echo "$3: generating '$1' from '$2'$option"
+
+    unset option
 }
 
-# Generate a file using several methods. A warning is automatically added to
-# the created file and a message printed to stdout through warning().
+# Generate a file using several methods. A warning not to edit it is
+# automatically added to the created file and a message printed to stdout
+# through warning().
 #
 # The following commands are possible; the file extension for the source file
 # in brackets.
 #
-# - m4  (.m4): pipe $2.m4 through m4 then write it to $2
-# - awk (.in): pipe $2.in through awk then write it to $2
-# - cat ($3):  copy $2$3 to $2
+# - m4   (.m4): pipe $2.m4 through m4 then write it to $2
+# - awk  (.in): pipe $2.in through awk then write it to $2
+# - perl (.in): pipe $2.in through perl then write it to $2
+# - cat  ($3):  copy $2$3 to $2
 generate() {
     # Get command and target file.
     command="$1"
@@ -105,7 +125,7 @@ generate() {
     # extension.
     if [ x"$command" = xm4 ]; then
         extension=.m4
-    elif [ x"$command" = xawk ]; then
+    elif [ x"$command" = xawk -o x"$command" = xperl ]; then
         extension=.in
     elif [ x"$command" = xcat ]; then
         extension="$1" # is $3 in reality, $1 because of shifting
@@ -117,7 +137,7 @@ generate() {
     fi
 
     # Add warning to file and write a message to stdout.
-    warning "$file" "$file$extension" $command
+    warning "$file" "$file$extension" $command "$*"
     # Generate $file from $file$extension using the given command.
     cat "$file$extension" | $command "$@" >> "$file"
@@ -4,19 +4,23 @@
 }
 
-# Prints the current OS. Supported are Debian (debian), Gentoo (gentoo) and
-# Mac OS X (darwin) at the moment. If an unsupported OS is used an error is
-# printed.
+# Print the current OS. The following OS are supported at the moment:
+# - Debian (debian)
+# - Gentoo (gentoo)
+# - Mac OS X (darwin)
+# - Solaris/OpenSolaris (sun)
+# - FreeBSD (freebsd)
+# If an unsupported OS is used an error is printed.
 os() {
     if [ -f /etc/debian_version ]; then